import cv2

def resize_pad(frame, new_shape=(640, 640)):
    ori_width, ori_height = frame.shape[:2][::-1]
    new_width, new_height = new_shape # 宽 高

    resize_rate = min(new_width / ori_width, new_height / ori_height)

    out_width = int(ori_width * resize_rate)
    out_height = int(ori_height * resize_rate)

    x_shift = (new_width - out_width) // 2
    y_shift = (new_height - out_height) // 2

    resized = cv2.resize(frame, (out_width, out_height))
    resized_padded = cv2.copyMakeBorder(resized, y_shift, y_shift, x_shift, x_shift, cv2.BORDER_CONSTANT, value=(0, 0, 0))

    roi = [[x_shift, y_shift], [x_shift + out_width, y_shift + out_height]]

    return resized_padded, roi

def matting(frame, roi):
    (x0, y0), (x1, y1) = roi
    frame = frame[y0:y1, x0:x1]
    return frame




import requests
from pathlib import Path
from . import b64

base_url = "" # API部署地址

def post(function, data, timeout):
    try:
        response = requests.post(url=f"{base_url}/{function}", json=data, timeout=timeout)
        if response.status_code != 200: 
            return False, f"请求失败！请求码：{response.status_code}；返回内容：{response.content.decode()}"
        response = response.json() # 解析返回内容 {"ret": True, "response": results}
        return response["ret"], response["response"]
    except Exception as e:
        return False, f"请求崩溃！崩溃原因：{str(e)}"

def status():
    ret, response = post(function="status", data={}, timeout=10)
    if not ret: return False, f"获取状态失败，失败原因：{response}"
    return True, response

def load(data):
    ret, response = post(function="load", data=data, timeout=120)
    if not ret: return False, f"模型加载失败，失败原因：{response}"
    return True, "模型加载成功"

def infer(data):
    ret, response = post(function="infer", data=data, timeout=1)
    if not ret: return False, f"图像推理失败，失败原因：{response}"
    return True, response


class GroundingDino:
    def __init__(self, weight_path, config_path, device="gpu"):
        self.weight_path = weight_path
        self.config_path = config_path
        self.device = device

        self.weight_name = Path(weight_path).name
        self.config_name = Path(config_path).name

        self.engine_name = "grounding_dino"
        self.engine_task = self.weight_name

        self.load() # 加载模型

    def load(self):
        ret, response = status() # 检查模型是否重复加载
        if not ret: return False, response
        if self.engine_task in response[self.engine_name]: return True, "模型已加载，无需重复加载"

        ret, response = load(data={ # 加载模型
            "engine_name": self.engine_name, 
            "engine_task": self.engine_task, 
            "arguments": {
                "weight": {"name": self.weight_name, "b64": b64.b_b64(open(self.weight_path, "rb").read())},
                "config": {"name": self.config_name, "b64": b64.b_b64(open(self.config_path, "rb").read())}, 
                "device": self.device
            }
        })
        return ret, response

    def infer(self, frame, conf=0.25, iou=0.7, text_prompt=False, text_threshold=0.25):
        ret, response = infer(data={
            "engine_name": self.engine_name, 
            "engine_task": self.weight_name, 
            "arguments": {
                "frame": {"b64": b64.cv_b64(frame)}, 
                "conf": conf, "iou": iou, 
                "text_prompt": text_prompt, "text_threshold": text_threshold
            }
        })
        return ret, response


class Yolov4:
    def __init__(self, weight_path, config_path, device="gpu"):
        self.weight_path = weight_path
        self.config_path = config_path
        self.device = device

        self.weight_name = Path(weight_path).name
        self.config_name = Path(config_path).name

        self.engine_name = "yolov4"
        self.engine_task = self.weight_name

        self.load() # 加载模型

    def load(self):
        ret, response = status() # 检查模型是否重复加载
        if not ret: return False, response
        if self.engine_task in response[self.engine_name]: return True, "模型已加载，无需重复加载"

        ret, response = load(data={ # 加载模型
            "engine_name": self.engine_name, 
            "engine_task": self.engine_task, 
            "arguments": {
                "weight": {"name": self.weight_name, "b64": b64.b_b64(open(self.weight_path, "rb").read())},
                "config": {"name": self.config_name, "b64": b64.b_b64(open(self.config_path, "rb").read())}, 
                "device": self.device
            }
        })
        return ret, response

    def infer(self, frame, conf=0.25, iou=0.7, classes=False):
        ret, response = infer(data={
            "engine_name": self.engine_name, 
            "engine_task": self.weight_name, 
            "arguments": {
                "frame": {"b64": b64.cv_b64(frame)}, 
                "conf": conf, "iou": iou, 
                "classes": classes
            }
        })
        return ret, response


class Yolov5:
    def __init__(self, weight_path, device="gpu", half=False):
        self.weight_path = weight_path
        self.device = device
        self.half = half

        self.weight_name = Path(weight_path).name

        self.engine_name = "yolov5"
        self.engine_task = self.weight_name

        self.load() # 加载模型

    def load(self):
        ret, response = status() # 检查模型是否重复加载
        if not ret: return False, response
        if self.engine_task in response[self.engine_name]: return True, "模型已加载，无需重复加载"

        ret, response = load(data={ # 加载模型
            "engine_name": self.engine_name, 
            "engine_task": self.engine_task, 
            "arguments": {
                "weight": {"name": self.weight_name, "b64": b64.b_b64(open(self.weight_path, "rb").read())},
                "device": self.device, 
                "half": self.half
            }
        })
        return ret, response

    def infer(self, frame, conf=0.25, iou=0.7, classes=False):
        ret, response = infer(data={
            "engine_name": self.engine_name, 
            "engine_task": self.weight_name, 
            "arguments": {
                "frame": {"b64": b64.cv_b64(frame)}, 
                "conf": conf, "iou": iou, 
                "classes": classes
            }
        })
        return ret, response


class Yolov5u:
    def __init__(self, weight_path, device="gpu", half=False):
        self.weight_path = weight_path
        self.device = device
        self.half = half

        self.weight_name = Path(weight_path).name

        self.engine_name = "yolov5u"
        self.engine_task = self.weight_name

        self.load() # 加载模型

    def load(self):
        ret, response = status() # 检查模型是否重复加载
        if not ret: return False, response
        if self.engine_task in response[self.engine_name]: return True, "模型已加载，无需重复加载"

        ret, response = load(data={ # 加载模型
            "engine_name": self.engine_name, 
            "engine_task": self.engine_task, 
            "arguments": {
                "weight": {"name": self.weight_name, "b64": b64.b_b64(open(self.weight_path, "rb").read())},
                "device": self.device, 
                "half": self.half
            }
        })
        return ret, response

    def infer(self, frame, conf=0.25, iou=0.7, classes=False):
        ret, response = infer(data={
            "engine_name": self.engine_name, 
            "engine_task": self.weight_name, 
            "arguments": {
                "frame": {"b64": b64.cv_b64(frame)}, 
                "conf": conf, "iou": iou, 
                "classes": classes
            }
        })
        return ret, response


class Yolov8u:
    def __init__(self, weight_path, device="gpu", half=False):
        self.weight_path = weight_path
        self.device = device
        self.half = half

        self.weight_name = Path(weight_path).name

        self.engine_name = "yolov8u"
        self.engine_task = self.weight_name

        self.load() # 加载模型

    def load(self):
        ret, response = status() # 检查模型是否重复加载
        if not ret: return False, response
        if self.engine_task in response[self.engine_name]: return True, "模型已加载，无需重复加载"

        ret, response = load(data={ # 加载模型
            "engine_name": self.engine_name, 
            "engine_task": self.engine_task, 
            "arguments": {
                "weight": {"name": self.weight_name, "b64": b64.b_b64(open(self.weight_path, "rb").read())},
                "device": self.device, 
                "half": self.half
            }
        })
        return ret, response

    def infer(self, frame, conf=0.25, iou=0.7, classes=False):
        ret, response = infer(data={
            "engine_name": self.engine_name, 
            "engine_task": self.weight_name, 
            "arguments": {
                "frame": {"b64": b64.cv_b64(frame)}, 
                "conf": conf, "iou": iou, 
                "classes": classes
            }
        })
        return ret, response




"""hekaiyue 何恺悦 2024-01-15"""
import copy


class IOUTracker:
    def __init__(self, threshold=0.5, buffer_size=5):
        self.threshold = threshold # 判断为同一目标的阈值
        self.buffer_size = buffer_size # 缓存大小
        self.buffer = list() # 缓存
        self.id = 0 # 追踪id

    def iou(self, box0, box1):
        # 计算交集的左上角和右下角坐标
        x0 = max(box0[0], box1[0])
        y0 = max(box0[1], box1[1])
        x1 = min(box0[2], box1[2])
        y1 = min(box0[3], box1[3])
        # 计算交集的宽度和高度
        intersection_width = max(0, x1 - x0)
        intersection_height = max(0, y1 - y0)
        # 计算交集面积
        intersection_area = intersection_width * intersection_height
        # 计算并集面积
        box0_area = (box0[2] - box0[0]) * (box0[3] - box0[1])
        box1_area = (box1[2] - box1[0]) * (box1[3] - box1[1])
        union_area = box0_area + box1_area - intersection_area
        if union_area <= 0: return 0
        # 计算交并比
        iou = intersection_area / union_area
        return iou

    def infer(self, results):
        results = copy.deepcopy(results)
        # [x0, y0, x1, y1, ...] --> [x0, y0, x1, y1, id, ...]
        now_ids = list()

        appear_ids = list()
        buffer = [_ for __ in self.buffer for _ in __] # 把缓存的三维列表展开成一维列表 [[[x0, y0, x1, y1, ...], ...], ...] 
        for current_target in results: # 遍历当前帧中所有目标
            current_id = False
            for history_target in buffer: # 遍历缓存帧中所有目标
                if self.iou(current_target, history_target) > self.threshold: # 如果 当前帧中的该目标 匹配到了 缓存帧中出现过的该目标
                    current_id = history_target[4] # 则 当前帧中的该目标 继承 缓存帧中出现过的该目标 的 追踪id
                    if current_id in now_ids: current_id = False # 如果已经被匹配掉了则不允许重复匹配
                    break
            if current_id is False: # 如果 当前帧中的该目标 未匹配到 缓存帧中出现过的任何目标
                self.id += 1
                current_id = self.id # 申请并赋予一个新id
                now_ids.append(current_id)
                current_target.insert(4, current_id)
                appear_ids.append(current_id) # 判定为新目标
            else: # 如果 当前帧中的该目标 匹配到 缓存帧中出现过的某个目标
                now_ids.append(current_id)
                current_target.insert(4, current_id) # 直接赋予继承到的 追踪id

        disappear_ids = list()
        self.buffer.append(results) # 将处理完成的当前帧推入缓存中
        if len(self.buffer) > self.buffer_size: # 如果缓存超长
            abort_buffer = self.buffer[0] # 获取头帧
            self.buffer = self.buffer[1:] # 抛弃头帧

            history_ids = [_[4] for __ in self.buffer for _ in __] # 查找并展平剩余buffer中所有帧中所有目标的id
            for abort_target in abort_buffer: # 遍历抛弃帧中所有被抛弃的目标
                abort_id = abort_target[4]
                if abort_id not in history_ids: # 如果被抛弃的目标的id没有再被继承的可能（连续buffer_size帧没有被继承过）
                    disappear_ids.append(abort_id) # 判定为消失目标
                    break
        
        return results, now_ids, appear_ids, disappear_ids



"""hekaiyue 何恺悦 2024-01-26"""
import cv2
import numpy
import base64

def cv_b64(frame_cv):
    frame_b64 = str(base64.b64encode(cv2.imencode(".jpg", frame_cv)[1]))[2:-1]
    return frame_b64

def b64_cv(frame_b64):
    frame_cv = cv2.imdecode(numpy.frombuffer(base64.b64decode(frame_b64), numpy.uint8), cv2.IMREAD_COLOR)
    return frame_cv

def b_b64(file_b):
    file_b64 = base64.b64encode(file_b).decode("utf-8")
    return file_b64

def b64_b(file_b64):
    file_b = base64.b64decode(file_b64)
    return file_b



from shapely.geometry import LineString

def crossed(line0, line1): # 两线段是否相交
    return LineString(line0).intersects(LineString(line1))

def vector(line): # 计算线段的向量
    (x0, y0), (x1, y1) = line
    return [x1-x0, y1-y0]

def clockwise_vector(line): # 计算线段顺时针转90度的向量
    x, y = vector(line)
    return [y, -x]

def acute(vector0, vector1): # 两向量夹角是否锐角
    x0, y0 = vector0
    x1, y1 = vector1
    dot_product = x0*x1 + y0*y1 # 如果点积小于0则判定为锐角 反之为钝角
    return dot_product >= 0



"""hekaiyue 何恺悦 2024-01-26"""
import os 
from setuptools import setup, find_packages

setup(
	name = "hky3535",
	version = "0.1.3",
    author ="hky3535",
    author_email = "hky3535@163.com",
	url = 'https://github.com/hky3535/hky3535.git',
    long_description_content_type="text/markdown",
	long_description = open('README.md',encoding="utf-8").read(),
    install_requires=[
        "opencv-python", 
        "numpy", 
        "pillow", 
        "tqdm", 
        "requests", 
        "shapely"
    ], 
	packages = find_packages(),
 	license = 'Apache',
   	classifiers = [
       'License :: OSI Approved :: Apache Software License'
    ],
    package_data={'': ['*.ttf']}, 
    include_package_data=True
)
